#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            smartWeb.py
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi), (based on code 
#                  initially provided by Antti Koivisto)
#
# Version:         0.1a
#
# Date:            11.04.2011
#
# Description:     Library to communicate with the SmartWeb module.
#                  (SmartWeb for Geopro/Oilon heat pumps using Siemens
#                  controller).
#                  
# Supported devices: 
#                  SmartWeb connected controllers.
#
# Requirements:    Python interpreter 2.4 or newer (www.python.org)
#                  (tested with 2.4.3)
#
# Version history: ** 11.04.2011 v0.1a (Olli Lammi) **
#                  First test version based on code written by
#                  Antti Koivisto. 
#
###########################################################################

# Imports

import sys, os 
import string
import re
import urllib, urllib2
from HTMLParser import HTMLParser

import log


###########################################################################

# Constants


###########################################################################

# Classes

class SessionIdParser(HTMLParser):
    def handle_starttag(self, tag, attrs):
        if tag=='input':
            idtag = [v for k,v in attrs if (k=='name' and v=='SessionId')]
            if idtag:            
                value = [v for k,v in attrs if k=='value']
                self.sessionId = str(value).strip("[']")


class SmartWeb(log.Logging):
    def __init__(self):
        log.Logging.__init__(self, 'SmartWeb')
        self.url = ''
        self.user = ''
        self.password = ''
        self.ids = {}
        self.sessionId = ''
        

    def setIds(self, idmap):
        self.ids = idmap


    def isAuth(self):
        if len(self.sessionId) > 0:
            return 1
        else:
            return 0


    def runQueryCommand(self, cmd):
        plantItemId = self.ids[cmd]
        if not self.isAuth():
            self.authenticate()
        
        res = self.fetchValue(plantItemId)
        if len(res) <= 0:
            if self.authenticate():                
                res = self.fetchValue(plantItemId)
        if len(res) <= 0:
            self.Log("ERROR: Unable to retrieve SmartWeb value for: " + cmd)
            return ""
        return res      


    def authenticate(self):
        self.sessionId = ''
        
        try: 
            f = urllib2.urlopen(self.url + "/main.app")
            data = f.read()
            f.close()

            parser = SessionIdParser()
            parser.feed(data)
            parser.close()
            self.sessionId = parser.sessionId
            self.Debug("Authenticated session %s" % self.sessionId)

            params = urllib.urlencode( { "user": self.user, "pwd": self.password } )
            tempurl = self.url + "/main.app?SessionId=" + self.sessionId + "&amp;section=auth"
            f = urllib2.urlopen(tempurl, params)
            #TODO check result
            f.close()
        except:
            self.sessionId = ''
        
        return self.isAuth()


    def fetchValue(self, plantItemId):
        params = {}
        params['SessionId'] = self.sessionId
        params['plantItemId'] = plantItemId
        params['service'] = 'getDp'
        params = urllib.urlencode(params)
        
        url = self.url + ("/ajax.app?%s" % params)
        self.Debug("Getting data using URL: %s" % url)

        data = ''
        try:
            f = urllib2.urlopen(url)
            data = f.read()
            f.close()
        except:
            data = ''

        if len(data) <= 0:
            self.Debug("ERROR: Cannot get data for item: %s" % plantItemId)
            return ""

        self.Debug("Retreived data: %s" % (repr(data)))

        result = getResultValue(data)

        if len(result) <= 0:
            self.Debug("ERROR: Retrieved data is invalid for item: %s" % plantItemId)
            return ""

        # FIXME! Here we assume that all data is float with 2 decimals
        try:
            result = "%.2f" % float(result)
        except:
            self.Debug("ERROR: Retrieved data value is invalid for item: %s" % plantItemId)
            result = ""        
        return result


###########################################################################

# Functions

def getResultValue(data):
    temp = dict(re.findall('"(?P<key>[^"]*?)"\s*:\s*"(?P<value>[^"]*?)"', data))
    
    if not temp.has_key('value'):
        return ""
    
    return string.strip(temp['value'])
