#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            sourceOumanEH800.py
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         0.1a
#
# Date:            20.01.2013
#
# Description:     TaloLogger configuration wrapper for OumanEH800 library.
#                  
# Version history: ** 20.01.2013 v0.1a (Olli Lammi) **
#                  First version. 
#
###########################################################################

# Imports

import os, string

import configuration
import dataSource
import oumanEH800


###########################################################################

# Classes

class OumanEH800Conf(oumanEH800.OumanEH800, configuration.Configurable, dataSource.DataSource):
    def __init__(self, conf, modname):
        configuration.Configurable.__init__(self)
        self.setModuleName(modname)
        oumanEH800.OumanEH800.__init__(self)
        dataSource.DataSource.__init__(self)
        self.setID(modname)
        self.handleConfiguration(conf)
        
    def getServiceName(self):
        return self.getModuleName()

    def handleConfiguration(self, conf): 
        self.URL = conf.getValue('URL', '', self.getModuleName())
        return (1, '')
                
    def initConfiguration(self):
        if len(self.URL) <= 0:
            return (-1, 'Missing Ouman EH800 URL.')
        if self.URL[len(self.URL)-1] != '/':
            self.URL = self.URL + '/' 
        
        measures = self.getMeasures(self.URL)
        if len(measures) <= 0:
            self.Log("Cannot initialize Ouman EH800 interface or no data points available.")
            return (0, '')

        str = 'Initialized Ouman EH800 module with datapoints:'
        for key in measures.keys():
            str = str + '\n    ' + key + ": " + measures[key][0]
        self.Log(str)
        return (1, '')

    def runDataSourceQueryCommandImpl(self, cmds):
        return self.runQueryCommand(cmds)

    @staticmethod
    def getAllowedConfigurationKeys():
        return (['URL'], [])
