#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            sourceTelldus.py
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.6a
#
# Date:            12.05.2014
#
# Description:     TaloLogger configuration wrapper for TelldusCore source.
#                  
# Version history: ** 12.05.2014 v1.6a (Olli Lammi) **
#                  Added to taloLogger. 
#
###########################################################################

# Imports

import string

import configuration
import dataSource
import telldusCore


###########################################################################

# Classes

class TelldusConf(telldusCore.TelldusSource, configuration.Configurable, dataSource.DataSource):
    def __init__(self, conf, modname):
        configuration.Configurable.__init__(self)
        self.setModuleName(modname)
        telldusCore.TelldusSource.__init__(self)
        dataSource.DataSource.__init__(self)
        self.setID(modname)
        self.handleConfiguration(conf)
        
    def getServiceName(self):
        return self.getModuleName()

    def handleConfiguration(self, conf):
        self.setDatapoints(self.handleTelldusConf(conf)) 
        return (1, '')

    def handleTelldusConf(self, conf):
        pointmap = {}
        for item in conf.getValue('SENSOR', [], self.getModuleName()):
            temps = string.split(item, ':', 5)
            if len(temps) < 5:
                self.log("ERROR: Invalid SENSOR configuration: " + item)
            else:
                try:
                    key = string.strip(temps[0])
                    protocol = string.strip(temps[1])
                    model = string.strip(temps[2])
                    id = int(string.strip(temps[3]))
                    datatype = string.upper(string.strip(temps[4]))
                except:
                    key = ''
                if len(key) <= 0:
                    self.log("ERROR: Invalid SENSOR configuration: " + item)
                else:
                    pointmap[key] = [protocol, model, id, datatype]
        return pointmap
            
    def initConfiguration(self):
        if len(self.DATAPOINTS.keys()) <= 0:
            return (0, 'No data points configured for TELLDUS datasource module.')

        if not self.initLibrary():        
            return (0, 'Unable to load libtelldus-core.so.2 library.')
        
        return (1, '')

    def releaseConfiguration(self):
        self.closeLibrary()
        return (1, '')

    def runDataSourceQueryCommandImpl(self, cmds):
        return self.runQueryCommand(cmds)

    @staticmethod
    def getAllowedConfigurationKeys():
        return ([], ['SENSOR'])
