#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            store.py
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.5a
#
# Date:            07.01.2013
#
# Description:     Abstract base class for services that can store logged 
#                  data values. 
#                 
# Requirements:    Python interpreter 2.4 or newer (www.python.org)
#                  (tested with 2.4.3)
# 
# Version history: ** 24.02.2009 v0.9a (Olli Lammi) **
#                  First beta version for testers. 
#
#                  ** 26.10.2011 v1.4f (Olli Lammi) **
#                  Added filtering features.
#
#                  ** 07.01.2013 v1.5a (Olli Lammi) **
#                  Added timeval parameter to logging. Added support for
#                  queuing data if storing fails.
#
###########################################################################

# Imports

import string

import threads
import configuration


###########################################################################

# Classes

class Store(configuration.Configurable):
    def __init__(self):
        configuration.Configurable.__init__(self)
        self.storefilters = None
        self.staged_data = []
        self.stage_lock = threads.Lock()
    
    def insertData(self, timeval, values):
        if self.storefilters != None:
            temp = []
            for value in values:
                if value[0] in self.storefilters:
                    temp.append(value)
            self.stageData(timeval, temp)
        else:
            self.stageData(timeval, values)
        self.handleStagedData()

    def stageData(self, timeval, values):
        if self.stage_lock.lock_wait():
            self.staged_data.append([timeval, values])
            self.stage_lock.free()

    def handleStagedData(self):
        if self.stage_lock.lock_wait():
            try:
                while len(self.staged_data) > 0:
                    item = self.staged_data[0]
                    if self.insertDataImpl(item[0], item[1]):
                        self.staged_data.pop(0)
                    else:
                        break
            except:
                pass
            self.stage_lock.free()

    def initStoreFilters(self, conf):
        self.storefilters = None
        temp = conf.getValue('STOREFILTER', [])
        if len(temp) > 0:
            tempfilters = []
            for filter in temp:
                if string.find(filter, ':') < 0:
                    return False
                [mname, posname] = string.split(filter, ':', 1)
                mname = string.strip(mname)
                posname = string.strip(posname)
                if mname == self.getModuleName() and posname not in tempfilters:
                    tempfilters.append(posname)
            if len(tempfilters) > 0:
                self.storefilters = tempfilters
        return True

    def checkStoreFilters(self, measurepositions):
        if self.storefilters != None:
            for filter in self.storefilters:
                if filter not in measurepositions:
                    return (0, "Data store module %s has invalid store filter %s." % (self.getModuleName(), filter))
        return (1, '')

    # returns 0 if inserting failed, 1 with success
    def insertDataImpl(self, timeval, values): pass
    